print("Bullet Roulette(1.1.1,MIT License)")
print("项目网址：https://pypi.org/project/bulletroulette/，开发者:GQX（PyPI 账号）")
print("在开发者模式（bulletroulette.dev的run函数）下运行\"readme()\"查看游戏玩法、鸣谢和其他说明，运行\"license()\"查看更详细的协议")
#print("Bullet Roulette(1.1.1)")
#print("MIT License")
#print("Run on Python 3.10.12, Pygame 2.5.2 for best results")
#print("Developer:GQX(PyPI account)")
#print("Enter \"readme()\"to show you that \"how to play\", \"thanks\" and \"other introduction\",run \"license()\" to show you license detail.")


# 外部导入
cannotuseGUI = False
from random import shuffle
from time import sleep
from os import chdir,path
from traceback import print_exc
from copy import copy
# 模块导入
from data import *
from sprites import *
# 以下是主程序
chdir(path.dirname(__file__))
health = (2,4,6)
buckshots = [ # 子弹（1代表实，0代表空）
        [
            [1,0,0],
            [1,1,0,0]
        ],
        [
            [1,0],
            [1,1,0,0,0],
            [1,1,1,1,0,0]
        ],
        [
            [1,1,1,1,1,0,0,0],
            [1,1,1,1,0,0,0,0],
            [1,1,1,0],
            [1,1,1,1,0,0,0,0]
        ]
    ]
def run(CHEAT = False):
    try:
        name = input("你的名字：") # 输入名字
        for j in range(3): # 三个回合
            dealer = Dealer(health[j]) # 恶魔初始化
            player = Player(health[j],name) # 玩家初始化
            for i in range(len(buckshots[j])): # 每一小轮
                buckshot = buckshots[j][i] # 初始化本轮子弹
                buckshotcount = [0,0] #子弹计数
                for k in buckshot: # 实弹
                    if k:buckshotcount[0]+=1
                for k in buckshot: # 空弹
                    if k == 0:buckshotcount[1]+=1
                next = 0 # 下一轮
                shuffle(buckshot) # 打乱子弹顺序
                dealer.setbullet(copy(buckshot))
                print(f"{buckshotcount[0]}发实弹，{buckshotcount[1]}发空弹") # 打印子弹提示
                for i in range(len(buckshot)): #
                    if CHEAT:
                        print("Bullets:",buckshot[i:])
                    print(f"恶魔血量:{dealer.gethealth()} {name}的血量:{player.gethealth()}")
                    if next == 0:
                        if player.shoot():
                            if buckshot[i]:
                                print("砰！！！")
                                dealer.hurt() # 受伤
                            else:
                                print("咔......")
                            next = 1
                        else:
                            if buckshot[i]:
                                print("砰！！！")
                                player.hurt()
                                next = 1
                            else:
                                print("咔......")
                    else:
                        if dealer.shoot():
                            print(f"恶魔选择向{name}开枪！！！")
                            if buckshot[i]:
                                print("砰！！！")
                                player.hurt()
                            else:
                                print("咔......")
                            next = 0
                        else:
                            print("恶魔选择向自己开枪......")
                            if buckshot[i]:
                                print("砰！！！")
                                dealer.hurt()
                                next = 0
                            else:
                                print("咔......")
                    if dealer.gethealth() == 0:
                        print("恶魔死了！！！")
                        break
                    if player.gethealth() == 0:
                        print(f"{name}死了！")
                        raise SystemExit
                    sleep(1.5)
            del dealer
            del player
    except KeyboardInterrupt:print("\n检测到^C")
    except SystemExit:pass
    except:
        print("抱歉，我们检测到了一个错误，这可能不是您造成的，但您无法继续进行游戏了")
        print("错误信息：")
        print_exc()
    print("游戏结束!")
    print("感谢您的游玩!")
    pygame.quit()


# 此文件用于声明类
import pygame
from data import *
from random import random

class Button:
    def __init__(self,x,y,image,mode = 0):
        self.image = image
        self.location = self.image.get_rect()
        if mode == 0:
            self.location.topleft = (x,y)
        else:
            self.location.center = (x,y)
            x = self.location.topleft[0]
            y = self.location.topleft[1]
        self.framelocation = pygame.Rect(x-5, y-5, self.image.get_width()+10, self.image.get_height()+10)
        self.clicked = False
    
    def run(self,screen):
        screen.blit(self.image,self.location)
        mouselocation = pygame.mouse.get_pos()
        if self.location.collidepoint(mouselocation):
            pygame.draw.rect(screen,RED,self.framelocation,5)
            if pygame.mouse.get_pressed()[0] == 1 and not self.clicked:
                self.clicked = True
                return 1
            else:
                self.clicked = False
                return 0

class Dealer: # 恶魔
    def __init__(self,health,prop = []) -> None: # 初始化
        self.__tophealth = health
        self.__health = health
        self.__prop = prop
        self.__value = None
        self.__bullet = []

    def gethealth(self) -> int:
        return self.__health

    def smoke(self) -> None:
        if self.__health < self.__tophealth:
            self.__health += 1

    def setprop(self,prop) -> None: # 抽取道具
        self.__prop.append(prop)

    def getprop(self) -> list:
        return self.__prop

    def shoot(self) -> int: # 开枪
        if len(self.__bullet) == 1:return self.__bullet[0]
        del self.__bullet[0]
        if self.__value != None:
            self.__value = None
            return self.__value
        if random() >= 0.5:return 1 # 1：向玩家开枪，0：向自己开枪
        else:return 0

    def noprop(self,rm = True) -> None:
        if rm:
            self.__prop = []
        else:
            while True:
                if len(self.__prop) > 8:
                    self.__prop.pop()
                    continue
                break

    def useprop(self) -> int: # 使用道具
        '''
        args:None
        return:
        （1：使用道具
            （
            0：不用）
        ）
        '''
        if not self.__prop:
            return 0
        while True:
            rdm = random()
            if rdm <= 0.19: # 啤酒
                if "beer" in self.__prop and len(self.__bullet) != 1 and self.__value != 1:
                    del self.__prop[self.__prop.index("beer")]
                    return 1
            elif rdm <= 0.38: # 烟
                if "smoke" in self.__prop and self.__health < self.__tophealth:
                    del self.__prop[self.__prop.index("smoke")]
                    return 2
            elif rdm <= 0.57: # 手铐
                if "handcuff" in self.__prop and len(self.__bullet) != 1:
                    del self.__prop[self.__prop.index("handcuff")]
                    return 3
            elif rdm <= 0.76: # 小刀
                if "knife" in self.__prop and self.__value == 1:
                    del self.__prop[self.__prop.index("knife")]
                    return 4
            elif rdm <= 0.95: # 放大镜
                if "magnifier" in self.__prop and len(self.__bullet) != 1 and self.__value == None:
                    del self.__prop[self.__prop.index("magnifier")]
                    return 5
            else:
                return 0

    def memory(self,value):
        self.__value = value

    def setbullet(self,bullet):
        self.__bullet = bullet

    def hurt(self):
        self.__health -= 1
        print("恶魔:啊！！！")

class Player: # 人（玩家）
    def __init__(self,health,name,prop = []) -> None: # 初始化（设置血量）
        self.__tophealth = health
        self.__health = health
        self.__name = name
        self.__prop = prop
    
    def smoke(self) -> None:
        if self.__health < self.__tophealth:
            self.__health += 1

    def shoot(self) -> int: # 打出
        return int(input(f"{self.__name}想射："))

    def gethealth(self) -> int:
        return self.__health

    def getprop(self) -> list:
        return self.__prop

    def useprop(self): # 抽取道具
        while True:
            using = input(f"使用道具：")
            if using in self.__prop:
                del self.__prop[self.__prop.index(using)]
                return using
            elif using == 0:
                return 0
            else:
                print("没有此道具")

    def setprop(self,prop) -> None: # 抽取道具
        self.__prop.append(prop)

    def hurt(self) -> int: # 受伤
        self.__health -= 1
        print(f"{self.__name}：啊！！！")

    def setname(self,name):
        self.__name = name


from locale import getdefaultlocale as lcl,setlocale as stlcl,LC_ALL
from gettext import translation
from random import shuffle
from time import sleep
import pygame
from copy import copy
from os.path import dirname
from sys import path
#import treading
#treading.Tread() # TODO
path.append(dirname(__file__))
# 模块导入
from data import *
from sprites import *

stlcl(LC_ALL,"")
lang, _ = lcl()
trans = translation("trans",LOCALE,languages=[lang],fallback=True)
_ = trans.gettext

def setprop(beset,value,prop):
    beset.setprop(value)
    prop.append(value)
    return prop

def makeprop(beset,value):
    prp = []
    for i in range(value):
        rdm = random()
        if rdm <= 0.2: # 啤酒
            prp = setprop(beset,"beer",prp)
        elif rdm <= 0.4: # 烟
            prp = setprop(beset,"smoke",prp)
        elif rdm <= 0.6: # 手铐
            prp = setprop(beset,"handcuff",prp)
        elif rdm <= 0.8: # 小刀
            prp = setprop(beset,"knife",prp)
        else:
            prp = setprop(beset,"magnifier",prp)
    return prp

def run(FPS = 15,CHEAT = False):
    try:
        pygame.init()
        # 以下是一些关于pygame的一些常量声明
        # 以下是初始化
        clock = pygame.time.Clock() # 初始化时钟
        screen = pygame.display.set_mode((1400,850)) # 屏幕（窗口）初始化
        pygame.display.set_caption("Bullet Roulette") # 设置标题
        text = pygame.font.Font(FONT,75) # 设定字体
        background    = pygame.image.load(BACKGROUND) # 加载背景图
        gunimage      = pygame.image.load(GUN) # 加载霰弹枪图片
        charge        = pygame.image.load(CHARGE) # 加载血量图片
        blank         = pygame.image.load(BLANK) # 加载空弹图片
        liveround     = pygame.image.load(LIVEROUND) # 加载实弹图片
        shootself     = pygame.image.load(SHOOTSELF) # 加载恶魔射击自己时的图片
        shootplayer   = pygame.image.load(SHOOTPLAYER) # 加载恶魔射击玩家时的图片
        smoke         = pygame.image.load(SMOKE) # 加载烟的图片
        magnifier     = pygame.image.load(MAGNIFIER) # 加载放大镜图片
        knife         = pygame.image.load(KNIFE) # 加载刀的图片
        handcuff      = pygame.image.load(HANDCUFF) # 加载手铐图片
        beer          = pygame.image.load(BEER) # 加载啤酒图片
        propbox       = pygame.image.load(PROPBOX) # 加载道具盒图片
        usesmoke      = pygame.image.load(USESMOKE) # 加载玩家使用烟时的图片
        usemagnifier  = pygame.image.load(USEMAGNIFIER) # 加载玩家使用放大镜时的图片
        usehandcuff   = pygame.image.load(USEHANDCUFF) # 加载玩家使用手铐时的图片
        usebeer       = pygame.image.load(USEBEER) # 加载玩家使用啤酒时的图片
        useknife      = pygame.image.load(USEKNIFE) # 加载玩家使用刀时的图片
        seeblank      = pygame.image.load(SEEBLANK) # 加载玩家使用放大镜看到空弹时的图片
        seeliveround  = pygame.image.load(SEELIVEROUND) # 加载玩家使用放大镜看到实弹时的图片
        dusesmoke     = pygame.image.load(DUSESMOKE) # 加载玩家使用烟时的图片
        dusemagnifier = pygame.image.load(DUSEMAGNIFIER) # 加载玩家使用放大镜时的图片
        dusehandcuff  = pygame.image.load(DUSEHANDCUFF) # 加载玩家使用手铐时的图片
        dusebeer      = pygame.image.load(DUSEBEER) # 加载玩家使用啤酒时的图片
        duseknife     = pygame.image.load(DUSEKNIFE) # 加载玩家使用刀时的图片
        nametext = text.render(_("输入名字："),True,WHITE) # 用字体生成文字
        playerlose = text.render(_("恶魔赢了！"),True,WHITE) # 用字体生成文字
        interestingtext = text.render(_("恶魔：非常有趣"),True,WHITE)
        shootdealer = Button(700,75,text.render(_("恶魔"),True,WHITE),1)
        dealerturntext = text.render(_("恶魔的回合"),True,WHITE)
        buckshottext = text.render(_("子弹："),True,WHITE)
        uselocation = usebeer.get_rect() # 获取使用道具的图片的位置
        seelocation = seeblank.get_rect()
        propboxlocation = propbox.get_rect()
        buckshotlocation = blank.get_rect() # 获取子弹显示位置
        shootlocation = shootself.get_rect() # 获取恶魔射击时的位置
        chargelocation = charge.get_rect() # 获取血量显示位置
        buckshottextlocation = buckshottext.get_rect()
        nametextlocation = nametext.get_rect()
        interestingtextlocation = interestingtext.get_rect()
        loseorwinlocation = playerlose.get_rect()
        seelocation.top = 0
        seelocation.centerx = 700
        uselocation.centerx = 700
        uselocation.bottom = 850 # 将使用道具的图片的位置的底边修改为700,850
        propboxbutton = Button(700,700,propbox,1)
        noneprop     = copy(knife)
        noneprop.set_alpha(0)
        proplocation = []
        propbuttons = []
        dproplocation = []
        for i in range(8):
            proplocation.append(pygame.Rect((100*(i+1)+200,425,50,100)))
        for i in range(8):
            propbuttons.append(Button(100*(i+1)+200,425,noneprop))
        for i in range(8):
            dproplocation.append(pygame.Rect((100*(i+1)+200,25,50,100)))
        gunimage.set_alpha(0) # 完全透明（此图是为了绘制边框）
        gun = Button(675,300,gunimage,1) # 加载按钮（Button类的定义在sprites.py)
        pygame.mixer.music.load(BGM) # 加载背景音乐（BGM）
        liveplayermusic = pygame.mixer.Sound(LIVEPLAYER) # 玩家被击中时的音效
        livedealermusic = pygame.mixer.Sound(LIVEDEALER) # 恶魔被击中时的音效
        blankmusic = pygame.mixer.Sound(BLANKMUSIC) # 空弹发射时的音效
        shootlocation.center = (700,300) # 设置恶魔射击时的位置
        interestingtextlocation.center = (700,600)
        loseorwinlocation.center = (700,425)
        propboxlocation.center = (700,700)
        nametextlocation.center = (700,100)
        buckshottextlocation.topleft = (100,75)
        dealerturntextlocation = copy(nametextlocation)
        dealerturntextlocation.topright = (1400,0)
        buttons = []
        name = ""
        for i in range(26):buttons.append(Button(50*(i+1),700,pygame.image.load(LETTERSPATH[i])))
        delete = Button(350,760,pygame.image.load(DELETE))
        enter = Button(800,760,pygame.image.load(ENTER))
        pygame.mixer.music.play(-1) # 循环播放音乐
        # 以下是一些控制变量
        first = True
        naming = True # 正在命名
        playerturn = False # 玩家回合
        dealerturn = False # 恶魔回合
        choosing = False # 玩家正在选择射击谁
        drawingbullets = False # 正在显示子弹和抽取道具
        selectingprop = True # 隶属于drawingbullets，正在选择道具（还没有点击道具盒）
        playercuff = False
        dealercuff = False
        playerknife = False
        dealerknife = False
        tmppropplayer = [] # 玩家当前将要抽取的道具
        propdealer = []
        propplayer = [None,None,None,None,None,None,None,None] # 玩家已经抽取的道具
        buckshot = [] # 当前小轮的子弹
        turn = [0,0] # 记录回合[大轮，小轮]
        dealer = Dealer(2) # 初始化Dealer类
        player = Player(2,name) # 初始化Player类
        count = 0
        while True: # 游戏主循环
            count += 1
            if CHEAT and count == FPS:
                count = 0
                print("Bullets:",buckshot)
            for event in pygame.event.get(): # 事件处理循环
                if event.type == pygame.QUIT: # QUIT处理
                    raise SystemExit # 退出(等同于exit())
            screen.blit(background,(0,0)) # 粘贴背景图片
            if player.gethealth() <= 0: # 玩家死亡
                screen.blit(playerlose,loseorwinlocation) # 打印玩家死亡提示
                pygame.display.update() # 更新画面
                sleep(2) # 停留2秒
                raise SystemExit # 退出
            if dealer.gethealth() <= 0: # 恶魔死亡
                turn[1] = 0 # 初始化小轮
                turn[0] += 1 # 大轮+1
                propplayer = [None,None,None,None,None,None,None,None]
                propdealer = []
                buckshot = [] # 初始化子弹
                screen.blit(playerwin,loseorwinlocation) # 打印恶魔死亡提示
                pygame.display.update() # 更新画面
                sleep(2) # 停留2秒
                if turn[0] == 3: # 超出3轮
                    raise SystemExit # 退出
                else:
                    dealer = Dealer(HEALTH[turn[0]]) # 初始化Dealer类
                    player = Player(HEALTH[turn[0]],name) # 初始化Player类
                    dealer.noprop()
            if not buckshot:
                buckshot = BULLETS[turn[0]][turn[1]]
                turn[1] += 1
                if turn[0]:
                    savepropdaler = copy(propdealer)
                    tmppropdealer = makeprop(dealer,HEALTH[turn[0] - 1])
                    propdealer = copy(savepropdaler)
                    propdealer.extend(tmppropdealer)
                    dealer.noprop(False)
                    propdealer = dealer.getprop()
                    tmppropplayer = makeprop(player,HEALTH[turn[0] - 1])
                if not naming:
                    playerturn = dealerturn = False
                    drawingbullets = True
            if naming:
                screen.blit(nametext,nametextlocation)
                for i in buttons: 
                    if i.run(screen):
                        name += (LETTERS[buttons.index(i)])
                if delete.run(screen):
                    name = name[:-1]
                if enter.run(screen):
                    naming = False
                    drawingbullets = True
                    shootyou = Button(700,700,text.render(name,False,WHITE),1)
                    playerwin = text.render(f"{name} 赢了！",False,WHITE)
                    player.setname(name)
                tmpnametext = text.render(name,False,WHITE)
                tmpnametextlocation = tmpnametext.get_rect()
                tmpnametextlocation.center = (700,400)
                screen.blit(tmpnametext,tmpnametextlocation)
            elif playerturn:
                for i in range(player.gethealth()):
                    chargelocation.bottomleft = (0,850-50*i)
                    screen.blit(charge,chargelocation)
                for i in range(dealer.gethealth()):
                    chargelocation.topleft = (0,0+50*i)
                    screen.blit(charge,chargelocation)
                for en,i in enumerate(propdealer):
                    screen.blit(eval(i),dproplocation[en])
                for en,i in enumerate(propplayer):
                    if i:
                        screen.blit(eval(i),proplocation[en])
                        if propbuttons[en].run(screen):
                            if i == "magnifier":screen.blit(usemagnifier,(600,620))
                            else:screen.blit(eval("use" + i),uselocation)
                            if i == "beer":
                                if buckshot[0]:
                                    screen.blit(liveround,(650,0))
                                else:
                                    screen.blit(blank,(650,0))
                                del buckshot[0]
                            elif i == "knife":
                                playerknife = True
                            elif i == "smoke":
                                player.smoke()
                            elif i == "handcuff":
                                dealercuff = True
                            else:
                                if buckshot[0]:
                                    screen.blit(seeliveround,seelocation)
                                else:
                                    screen.blit(seeblank,seelocation)
                            propplayer[en] = None
                            pygame.display.update()
                            sleep(2)
                if gun.run(screen):
                    playerturn = False
                    choosing = True
            elif dealerturn:
                screen.blit(dealerturntext,dealerturntextlocation)
                for i in range(player.gethealth()):
                    chargelocation.bottomleft = (0,850-50*i)
                    screen.blit(charge,chargelocation)
                for i in range(dealer.gethealth()):
                    chargelocation.topleft = (0,0+50*i)
                    screen.blit(charge,chargelocation)
                for en,i in enumerate(propdealer):
                    screen.blit(eval(i),dproplocation[en])
                for en,i in enumerate(propplayer):
                    if i:screen.blit(eval(i),proplocation[en])
                pygame.display.update()
                sleep(1)
                while True:
                    if not dealer.getprop():break
                    useprop = dealer.useprop()
                    if useprop == 0:
                        break
                    elif useprop == 1: # 啤酒
                        screen.blit(dusebeer,seelocation)
                        if buckshot[0]:
                            screen.blit(liveround,uselocation)
                        else:
                            screen.blit(blank,uselocation)
                        del buckshot[0]
                    elif useprop == 2: # 烟
                        screen.blit(dusesmoke,seelocation)
                        dealer.smoke()
                    elif useprop == 3:
                        screen.blit(dusehandcuff,seelocation)
                        playercuff = True
                    elif useprop == 4:
                        screen.blit(duseknife,seelocation)
                        dealerknife = True
                    else:
                        screen.blit(dusemagnifier,seelocation)
                        if buckshot[0]:
                            dealer.memory(1)
                        else:
                            dealer.memory(0)
                        screen.blit(interestingtext,interestingtextlocation)
                    propdealer = dealer.getprop()
                    pygame.display.update()
                    sleep(2)
                    screen.blit(background,(0,0))
                    screen.blit(dealerturntext,dealerturntextlocation)
                    for i in range(player.gethealth()):
                        chargelocation.bottomleft = (0,850-50*i)
                        screen.blit(charge,chargelocation)
                    for i in range(dealer.gethealth()):
                        chargelocation.topleft = (0,0+50*i)
                        screen.blit(charge,chargelocation)
                    for en,i in enumerate(propdealer):
                        screen.blit(eval(i),dproplocation[en])
                    for en,i in enumerate(propplayer):
                        if i:screen.blit(eval(i),proplocation[en])
                    pygame.display.update()
                    sleep(1)
                if buckshot:
                    if dealer.shoot() == 0:
                        screen.blit(shootself,shootlocation)
                        if buckshot[0]:
                            livedealermusic.play()
                            dealer.hurt()
                            if not playercuff:
                                dealerturn = False
                                playerturn = True
                            else:playercuff = False
                        else:
                            blankmusic.play()
                    else:
                        screen.blit(shootplayer,shootlocation)
                        if buckshot[0]:
                            liveplayermusic.play()
                            if dealerknife:
                                player.hurt()
                            player.hurt()
                        else:
                            blankmusic.play()
                        if not playercuff:
                            dealerturn = False
                            playerturn = True
                        else: playercuff = False
                    print("deling",buckshot)
                    del buckshot[0]
                    pygame.display.update()
                    sleep(2)
                else:pass
            elif choosing:
                for i in range(player.gethealth()):
                    chargelocation.bottomleft = (0,850-50*i)
                    screen.blit(charge,chargelocation)
                for i in range(dealer.gethealth()):
                    chargelocation.topleft = (0,0+50*i)
                    screen.blit(charge,chargelocation)
                if shootdealer.run(screen):
                    choosing = False
                    if dealercuff:
                        playerturn = True
                        dealercuff = False
                    else:
                        dealerturn = True
                    if buckshot[0] == 0:
                        blankmusic.play()
                    else:
                        livedealermusic.play()
                        dealer.hurt()
                        if playerknife:
                            dealer.hurt()
                    del buckshot[0]
                    playerknife = False
                    sleep(2)
                elif shootyou.run(screen):
                    choosing = False
                    if buckshot[0] == 0:
                        blankmusic.play()
                        playerturn = True
                    else:
                        liveplayermusic.play()
                        player.hurt()
                        if playerknife:
                            player.hurt()
                        if dealercuff:
                            playerturn = True
                            dealercuff = False
                        else:
                            dealerturn = True
                    del buckshot[0]
                    playerknife = False
                    sleep(2)
            elif drawingbullets:
                pygame.draw.rect(screen,RED,(0,0,1400,250))
                screen.blit(buckshottext,buckshottextlocation)
                for en,b in enumerate(buckshot):
                    buckshotlocation.topleft = (600+80 * en,75)
                    if b:
                        screen.blit(liveround,buckshotlocation)
                    else:
                        screen.blit(blank,buckshotlocation)
                for en,p in enumerate(propplayer):
                    if p:
                        screen.blit(eval(p),proplocation[en])
                if tmppropplayer:
                    screen.blit(propbox,propboxlocation)
                    first = False
                    if selectingprop:
                        if propboxbutton.run(screen):
                            selectingprop = False
                    else:
                        screen.blit(eval(tmppropplayer[0]),propboxlocation)
                        for en,p in enumerate(propbuttons):
                            if not propplayer[en] and p.run(screen):
                                selectingprop = True
                                propplayer[en] = tmppropplayer[0]
                                del tmppropplayer[0]
                else:
                    if first:
                        pygame.display.update()
                        sleep(2)
                    else:
                        first = True
                    drawingbullets = False
                    playerturn = True
                    shuffle(buckshot)
                    dealer.setbullet(copy(buckshot))
 #           else:
#                raise TypeError("Nothing is Running!!!")
            pygame.display.update()
            clock.tick(FPS)
    except KeyboardInterrupt:pass
    except SystemExit:pass
    pygame.quit()


# 此文件用于定义各种常量。
from os.path import dirname

ASSESTS       = f"{dirname(__file__)}/assets/" # 存放图片背景音乐等素材的目录
LOCALE        = f"{dirname(__file__)}/locale/" # 存放翻译文件的路径
RED           = (255,  0,  0) # 红色RGB
WHITE         = (255,255,255) # 白色RGB
BACKGROUND    = f"{ASSESTS}bg.png" # 背景图
BGM           = f"{ASSESTS}bgmsc.ogg" # 背景音乐
FONT          = f"{ASSESTS}Simhei.ttf" # 字体
LETTERS       = ("A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z") # 字母表
DELETE        = f"{ASSESTS}DELETE.png" # DELETE键
ENTER         = f"{ASSESTS}ENTER.png" # ENTER键
CHARGE        = f"{ASSESTS}charge.png" # 电量
GUN           = f"{ASSESTS}gun.png" # 枪（轮廓）
BLANKMUSIC    = f"{ASSESTS}blank.wav" # 射出空弹音效
LIVEDEALER    = f"{ASSESTS}livedealer.wav" # 大哥被打到时的音效
LIVEPLAYER    = f"{ASSESTS}liveplayer.wav" # 玩家被打到时的音效
BLANK         = f"{ASSESTS}blank.png" # 空弹
LIVEROUND     = f"{ASSESTS}liveround.png" # 实弹
SHOOTSELF     = f"{ASSESTS}dealershootself.png" # 大哥紫砂
SHOOTPLAYER   = f"{ASSESTS}dealershootplayer.png" # 大哥打玩家
SMOKE         = f"{ASSESTS}smoke.png"
HANDCUFF      = f"{ASSESTS}handcuff.png"
MAGNIFIER     = f"{ASSESTS}magnifier.png"
KNIFE         = f"{ASSESTS}knife.png"
PROPBOX       = f"{ASSESTS}propbox.png"
BEER          = f"{ASSESTS}beer.png"
PILL          = f"{ASSESTS}pill.png"
NONEPROP      = f"{ASSESTS}noneprop.png"
USESMOKE      = f"{ASSESTS}usesmoke.png"
USEKNIFE      = f"{ASSESTS}useknife.png"
USEHANDCUFF   = f"{ASSESTS}usehandcuff.png"
USEBEER       = f"{ASSESTS}usebeer.png"
USEMAGNIFIER  = f"{ASSESTS}usemagnifier.png"
DUSESMOKE     = f"{ASSESTS}dusesmoke.png"
DUSEKNIFE     = f"{ASSESTS}duseknife.png"
DUSEHANDCUFF  = f"{ASSESTS}dusehandcuff.png"
DUSEBEER      = f"{ASSESTS}dusebeer.png"
DUSEMAGNIFIER = f"{ASSESTS}dusemagnifier.png"
SEEBLANK      = f"{ASSESTS}seeblank.png"
SEELIVEROUND  = f"{ASSESTS}seeliveround.png"
BOOM          = f"{ASSESTS}boom.png"
HEALTH        = (2,4,6)
BULLETS = [ # 子弹（1代表实，0代表空）
        [
            [1,0,0],
            [1,1,0,0]
        ],
        [
            [1,0],
            [1,1,0,0,0],
            [1,1,1,0,0,0],
            [1,1,1,1,0,0,0,0]
        ],
        [
            [1,1,1,1,1,0,0,0],
            [1,1,1,1,0,0,0,0],
            [1,1,1,1,1,1,0,0],
            [1,1,1,1,0,0,0,0],
            [1,1,0,0,0,0,0,0]
        ]
    ]
LETTERSPATH = [ASSESTS + l + ".png" for l in LETTERS]


# 主程序
# 外部导入
from locale import getdefaultlocale as lcl,setlocale as stlcl,LC_ALL
from gettext import translation
from random import shuffle,random
from time import sleep
from os.path import dirname
from sys import path
#import treading
#treading.Tread() # TODO
path.append(dirname(__file__))
# 模块导入
from data import *
from sprites import Dealer,Player

stlcl(LC_ALL,"")
lang, _ = lcl()
trans = translation("trans",LOCALE,languages=[lang],fallback=True)
_ = trans.gettext

def printprop(prop):
    for p in prop:
        if p == "beer":
            print(_("啤酒"))
        elif p == "smoke":
            print(_("烟"))
        elif p == "handcuff":
            print(_("手铐"))
        elif p == "knife":
            print(_("小刀"))
        else:
            print(_("放大镜"))

# 以下是主程序
def run(CHEAT = False):
    try:
        playerknife = False
        dealerknife = False
        name = input(_("输入名字：")) # 输入名字
        for j in range(3): # 三个回合
            brk = False
            dealer = Dealer(HEALTH[j]) # 恶魔初始化
            player = Player(HEALTH[j],name) # 玩家初始化
            for i in range(len(BULLETS[j])): # 每一小轮
                if brk:
                    brk = False
                    break
                buckshot = BULLETS[j][i] # 初始化本轮子弹
                buckshotcount = [0,0] #子弹计数
                for k in buckshot: # 实弹
                    if k:buckshotcount[0]+=1
                for k in buckshot: # 空弹
                    if k == 0:buckshotcount[1]+=1
                shuffle(buckshot) # 打乱子弹顺序
                dealer.setbullet(buckshot)
                print(_(f"{buckshotcount[0]}发实弹,{buckshotcount[1]}发空弹")) # 打印子弹提示
                if j:
                    for k in range(HEALTH[j-1]):
                        rdm = random()
                        if rdm <= 0.2: # 啤酒
                            dealer.setprop("beer")
                        elif rdm <= 0.4: # 烟
                            dealer.setprop("smoke")
                        elif rdm <= 0.6: # 手铐
                            dealer.setprop("handcuff")
                        elif rdm <= 0.8: # 小刀
                            dealer.setprop("knife")
                        else:
                            dealer.setprop("magnifier")
                    for k in range(HEALTH[j-1]):
                        rdm = random()
                        if rdm <= 0.2: # 啤酒
                            player.setprop("beer")
                        elif rdm <= 0.4: # 烟
                            player.setprop("smoke")
                        elif rdm <= 0.6: # 手铐
                            player.setprop("handcuff")
                        elif rdm <= 0.8: # 小刀
                            player.setprop("knife")
                        else:
                            player.setprop("magnifier")
                print(_("恶魔的道具："))
                printprop(dealer.getprop())
                print(_("你的道具："))
                printprop(player.getprop())
                next = 0 # 下一轮
                brk = False
                ctn = False
                ctnl = False
                playercuff = False
                dealercuff = False
                for b in buckshot:
                    if CHEAT:
                        print("Bullets:",buckshot)
                    print(f"恶魔血量：{dealer.gethealth()} {name}的血量：{player.gethealth()}")
                    if next == 0:
                        if j:
                            while True:
                                useprop = player.useprop()
                                if useprop == 0: # TODO
                                    break
                                elif useprop == 1: # 啤酒
                                    print(_("正在使用：啤酒"))
                                    if b:
                                        print(_("下一发是：实弹"))
                                    else:
                                        print(_("下一发是：空弹"))
                                    ctnl = True
                                    break
                                elif useprop == 2: # 烟
                                    print(_("正在使用：烟"))
                                    player.smoke()
                                elif useprop == 3:
                                    print(_("正在使用：手铐"))
                                    dealercuff = True
                                elif useprop == 4:
                                    print(_("正在使用：小刀"))
                                    playerknife = True
                                else:
                                    print(_("正在使用：放大镜"))
                                    if buckshot[0]:
                                        print(_("下一发是：实弹"))
                                    else:
                                        print(_("下一发是：空弹"))
                            if ctnl:
                                ctnl = False
                                continue
                        if player.shoot(): # 想射#TODO
                            if b:
                                print(_("砰！！！"))
                                dealer.hurt() # 受伤
                                if playerknife:
                                    dealer.hurt() # 受伤
                            else:
                                print(_("咔......"))
                            if not dealercuff:next = 1
                            else:dealercuff = False
                        else:
                            if b:
                                print(_("砰！！！"))
                                player.hurt()
                                if playerknife:
                                    player.hurt()
                                if not dealercuff:next = 1
                                else:dealercuff = False
                            else:
                                print(_("咔......"))
                    else:
                        if j:
                            while True:
                                useprop = dealer.useprop()
                                if useprop == 0:
                                    break
                                elif useprop == 1: # 啤酒
                                    print(_("恶魔正在使用：啤酒"))
                                    if b:
                                        print(_("下一发是：实弹"))
                                    else:
                                        print(_("下一发是：空弹"))
                                    ctnl = True
                                    break
                                elif useprop == 2: # 烟
                                    print(_("恶魔正在使用：烟"))
                                    dealer.smoke()
                                elif useprop == 3:
                                    print(_("恶魔正在使用：手铐"))
                                    playercuff = True
                                elif useprop == 4:
                                    print(_("恶魔正在使用：小刀"))
                                    dealerknife = True
                                else:
                                    print(_("恶魔正在使用：放大镜")) # TODO
                                    print(_("恶魔:非常有趣。。。"))
                            if ctnl:
                                ctnl = False
                                continue
                        if dealerknife:
                            print(_("恶魔选择向你开枪！！！"))
                            if b:
                                print(_("砰！！！"))
                                player.hurt()
                                player.hurt()
                            else:
                                print(_("咔......"))
                            if not playercuff:next = 0
                            else:playercuff = False
                            dealerknife = False
                        else:
                            if dealer.shoot():
                                print(_("恶魔选择向你开枪！！！"))
                                if b:
                                    print(_("砰！！！"))
                                    player.hurt()
                                else:
                                    print(_("咔......"))
                                if not playercuff:next = 0
                                else:playercuff = False
                            else:
                                print(_("恶魔选择向自己开枪......"))
                                if b:
                                    print(_("砰！！！"))
                                    dealer.hurt()
                                    if not playercuff:next = 0
                                    else:playercuff = False
                                else:
                                    print(_("咔......"))
                    playerknife = False
                    sleep(1)
                    if dealer.gethealth() == 0:
                        print("恶魔死了！！！")
                        brk = True
                        break
                    if player.gethealth() == 0:
                        print(f"{name}死了！！！")
                        raise SystemExit
            del dealer
            del player
    except KeyboardInterrupt:pass


from os.path import dirname

with open(f"{dirname(__file__)}/../README.md", "r", encoding="utf-8") as fh:
    readmetxt = fh.read()
with open(f"{dirname(__file__)}/../LICENSE", "r", encoding="utf-8") as fh:
    licensetxt = fh.read()

def run():
    def readme():print(readmetxt)
    def license():print(licensetxt)
    while True:
        exec(input("(roulette)>>> "))


from setuptools import setup,find_packages
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
setup(
    name="bulletroulette",  # 模块名称
    version="1.1.1",  # 当前版本
    author="GQX",  # 作者
    author_email="kill114514251@outlook.com",  # 作者邮箱
    description="Imitate game \"Buskshot Roulette\"",  # 模块简介
    long_description=long_description,  # 模块详细介绍
    long_description_content_type="text/markdown",  # 模块详细介绍格式
    url="https://github.com/BinaryGuo/Bullet_Roulette",  # 模块github地址
    packages=find_packages(),  # 自动找到项目中导入的模块
    package_data={
        "bulletroulette" : ["assets/*.png","assets/*.ogg","assets/*.ttf","assets/*.wav"]
    },
    # 模块相关的元数据
    classifiers=[
        "Intended Audience :: End Users/Desktop",
        "Development Status :: 3 - Alpha",
        "Topic :: Software Development :: Libraries :: pygame",
        "Natural Language :: Chinese (Simplified)",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.8"
    ],
    # 依赖模块
    install_requires=[
        "pygame>=2.0.1",
    ],
    python_requires=">=3.8"
)